#include <cmath>

#include "MathLib.h"

// Arithmetic functions
int MLSquare (int iArg_)
{
	return (iArg_ * iArg_);
}

int MLSquareroot (int iArg_)
{
	return ((int)sqrt ((double)iArg_));
}

float MLSquare (float fArg_)
{
	return (fArg_ * fArg_);
}

float MLSquareroot (float fArg_)
{
	return (sqrtf (fArg_));
}

// Trigonometric functions
float MLSin (float fArg_)
{
	fArg_ = fArg_ * (MLPI / 180.0f);

	return (sinf (fArg_));
}

float MLTan (float fArg_)
{
	fArg_ = fArg_ * (MLPI / 180.0f);

	return (tanf (fArg_));
}

float MLCos (float fArg_)
{
	fArg_ = fArg_ * (MLPI / 180.0f);

	return (cosf (fArg_));
}

// Vector functions
MLVector3::MLVector3 ()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}

MLVector3::MLVector3 (float x_, float y_, float z_)
{
	x = x_;
	y = y_;
	z = z_;
}

float MLVector3::Mag ()
{
	return (MLSquareroot (MLSquare (x) + MLSquare (y) + MLSquare (z)));
}

void MLVector3::Normalize ()
{
	float fMag = Mag ();

	x = x / fMag;
	y = y / fMag;
	z = z / fMag;
}

float MLVector3::DotProduct (MLVector3 &vArg_)
{
	return ((x * vArg_.x) + (y * vArg_.y) + (z * vArg_.z));
}

MLVector3 MLVector3::CrossProduct (MLVector3 &vArg_)
{
	MLVector3 Result;

	Result.x = (y * vArg_.z) - (z * vArg_.y);
	Result.y = (z * vArg_.x) - (x * vArg_.z);
	Result.z = (x * vArg_.y) - (y * vArg_.x);

	return (Result);
}

MLVector3 MLVector3::operator + (MLVector3 vArg_)
{
	MLVector3 Result;

	Result.x = x + vArg_.x;
	Result.y = y + vArg_.y;
	Result.z = z + vArg_.z;

	return (Result);
}

MLVector3 MLVector3::operator - (MLVector3 vArg_)
{
	MLVector3 Result;

	Result.x = x - vArg_.x;
	Result.y = y - vArg_.y;
	Result.z = z - vArg_.z;

	return (Result);
}

MLVector3 MLVector3::operator = (MLVector3 vArg_)
{
	x = vArg_.x;
	y = vArg_.y;
	z = vArg_.z;

	return (*this);
}

MLVector3 MLVector3::operator * (float fArg_)
{
	MLVector3 Result;

	Result.x = x * fArg_;
	Result.y = y * fArg_;
	Result.z = z * fArg_;

	return (Result);
}

MLVector3 MLVector3::operator / (float fArg_)
{
	MLVector3 Result;

	Result.x = x / fArg_;
	Result.y = y / fArg_;
	Result.z = z / fArg_;

	return (Result);
}